import { editor, NoOptionsHintView, buttonStyles } from 'pageflow-scrolled/editor';
import { FileInputView, CheckBoxInputView, ColorInputView as ColorInputView$1, editor as editor$1, transientReferences, ListView, SliderInputView as SliderInputView$1, SelectInputView as SelectInputView$1, EnumTableCellView } from 'pageflow/editor';
import { SelectInputView, ColorInputView, TextInputView, SliderInputView, CheckBoxInputView as CheckBoxInputView$1, UrlInputView, ConfigurationEditorView, cssModulesUtils } from 'pageflow/ui';
import Marionette from 'backbone.marionette';
import I18n from 'i18n-js';
import Backbone from 'backbone';
import _ from 'underscore';

editor.contentElementTypes.register('heading', {
  supportedPositions: ['inline', 'wide'],
  defaultConfig: {
    position: 'wide'
  },
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.group('ContentElementPosition');
    });
  }
});

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

editor.contentElementTypes.register('textBlock', {
  supportedPositions: ['inline'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.view(NoOptionsHintView);
    });
  },
  split: function split(configuration, insertIndex) {
    return [_objectSpread2(_objectSpread2({}, configuration), {}, {
      value: configuration.value.slice(0, insertIndex)
    }), _objectSpread2(_objectSpread2({}, configuration), {}, {
      value: configuration.value.slice(insertIndex)
    })];
  },
  merge: function merge(configurationA, configurationB) {
    // Value might still be empty if text block has not been edited
    var value = (configurationA.value || []).concat(configurationB.value || []);
    return _objectSpread2(_objectSpread2({}, configurationA), {}, {
      // Slate.js does not like empty arrays as value.
      // `inlineEditing/EditableText` sets default value, but only if
      // `value` is falsy.
      value: value.length ? value : undefined
    });
  },
  handleDestroy: function handleDestroy(contentElement) {
    var transientState = contentElement.get('transientState') || {};

    if (!transientState.editableTextIsSingleBlock) {
      contentElement.postCommand({
        type: 'REMOVE'
      });
      return false;
    }
  }
});

editor.contentElementTypes.register('inlineImage', {
  supportedPositions: ['inline', 'sticky', 'left', 'right', 'wide', 'full'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.input('id', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false
      });
      this.group('ContentElementCaption');
      this.group('ContentElementPosition');
    });
  }
});

editor.contentElementTypes.register('inlineVideo', {
  supportedPositions: ['inline', 'sticky', 'left', 'right', 'full'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.input('id', FileInputView, {
        collection: 'video_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false,
        defaultTextTrackFilePropertyName: 'defaultTextTrackFileId'
      });
      this.input('posterId', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false
      });
      this.input('autoplay', CheckBoxInputView);
      this.input('atmoDuringPlayback', SelectInputView, {
        values: ['play', 'mute', 'turnDown']
      });
      this.group('ContentElementCaption');
      this.group('ContentElementPosition');
    });
  }
});

editor.contentElementTypes.register('inlineAudio', {
  supportedPositions: ['inline', 'sticky', 'left', 'right', 'full'],
  configurationEditor: function configurationEditor(_ref) {
    var entry = _ref.entry;
    this.tab('general', function () {
      this.input('id', FileInputView, {
        collection: 'audio_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false,
        defaultTextTrackFilePropertyName: 'defaultTextTrackFileId'
      });
      this.input('posterId', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false
      });
      this.input('autoplay', CheckBoxInputView);
      this.input('playerControlVariant', SelectInputView, {
        values: ['classic', 'waveform'],
        ensureValueDefined: true
      });
      this.input('waveformColor', ColorInputView, {
        visibleBinding: 'playerControlVariant',
        visibleBindingValue: 'waveform',
        defaultValue: entry.getTheme().get('options').colors.accent
      });
      this.input('atmoDuringPlayback', SelectInputView, {
        values: ['play', 'mute', 'turnDown']
      });
      this.group('ContentElementCaption');
      this.group('ContentElementPosition');
    });
  }
});

editor.contentElementTypes.register('inlineBeforeAfter', {
  supportedPositions: ['inline'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.input('before_id', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false
      });
      this.input('before_label', TextInputView);
      this.input('after_id', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElementConfiguration',
        positioning: false
      });
      this.input('after_label', TextInputView);
      this.input('initial_slider_position', SliderInputView);
      this.input('slider', CheckBoxInputView$1);
      this.input('slider_handle', CheckBoxInputView$1, {
        visibleBinding: 'slider'
      });
      this.input('slider_color', ColorInputView$1, {
        visibleBinding: 'slider'
      });
      this.group('ContentElementPosition');
    });
  },
  defaultConfig: {
    slider: true,
    slider_handle: true,
    initial_slider_position: 50
  }
});

editor.contentElementTypes.register('soundDisclaimer', {
  supportedPositions: ['inline'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.view(NoOptionsHintView);
    });
  }
});

editor.contentElementTypes.register('videoEmbed', {
  supportedPositions: ['inline', 'sticky', 'left', 'right', 'full'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.input('videoSource', UrlInputView, {
        supportedHosts: ['http://youtu.be', 'https://youtu.be', 'http://www.youtube.com', 'https://www.youtube.com', 'http://vimeo.com', 'https://vimeo.com', 'http://www.facebook.com', 'https://www.facebook.com'],
        displayPropertyName: 'displayVideoSource',
        required: true,
        permitHttps: true
      });
      this.input('hideInfo', CheckBoxInputView$1);
      this.input('hideControls', CheckBoxInputView$1);
      this.input('aspectRatio', SelectInputView, {
        values: ['wide', 'narrow', 'square', 'portrait']
      });
      this.group('ContentElementCaption');
      this.group('ContentElementPosition');
    });
  }
});

var SidebarRouter = Marionette.AppRouter.extend({
  appRoutes: {
    'scrolled/external_links/:id/:link_id': 'link'
  }
});

var SidebarEditLinkView = Marionette.Layout.extend({
  template: function template(data) {
    return "\n    <a class=\"back\">".concat(I18n.t('pageflow_scrolled.editor.content_elements.externalLinkList.back'), "</a>\n    <a class=\"destroy\">").concat(I18n.t('pageflow_scrolled.editor.content_elements.externalLinkList.destroy'), "</a>\n\n    <div class='form_container'></div>\n  ");
  },
  className: 'edit_external_link',
  regions: {
    formContainer: '.form_container'
  },
  events: {
    'click a.back': 'goBack',
    'click a.destroy': 'destroyLink'
  },
  initialize: function initialize(options) {},
  onRender: function onRender() {
    var configurationEditor = new ConfigurationEditorView({
      model: this.model,
      attributeTranslationKeyPrefixes: ['pageflow_scrolled.editor.content_elements.externalLinkList.attributes'],
      tabTranslationKeyPrefix: 'pageflow_scrolled.editor.content_elements.externalLinkList.tabs'
    });
    var self = this;
    configurationEditor.tab('edit_link', function () {
      this.input('url', TextInputView, {
        required: true
      });
      this.input('open_in_new_tab', CheckBoxInputView$1);
      this.input('thumbnail', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElement.externalLinks.link',
        fileSelectionHandlerOptions: {
          contentElementId: self.options.contentElement.get('id')
        },
        positioning: false
      });
      this.input('title', TextInputView, {
        required: true
      });
      this.input('description', TextInputView);
    });
    this.formContainer.show(configurationEditor);
  },
  goBack: function goBack() {
    editor$1.navigate("/scrolled/content_elements/".concat(this.options.contentElement.get('id')), {
      trigger: true
    });
  },
  destroyLink: function destroyLink() {
    if (window.confirm(I18n.t('pageflow_scrolled.editor.content_elements.externalLinkList.confirm_delete_link'))) {
      this.options.collection.remove(this.model);
      this.goBack();
    }
  }
});

var ExternalLinkModel = Backbone.Model.extend({
  modelName: 'ExternalLink',
  i18nKey: 'external_link',
  mixins: [transientReferences],
  thumbnailUrl: function thumbnailUrl() {
    var image = this.collection.entry.imageFiles.getByPermaId(this.get('thumbnail'));
    return image ? image.get('thumbnail_url') : '';
  },
  title: function title() {
    return this.get('title');
  }
});

var ExternalLinkCollection = Backbone.Collection.extend({
  model: ExternalLinkModel,
  initialize: function initialize(models, options) {
    this.entry = options.entry;
    this.configuration = options.configuration;
    this.bind('change', this.updateConfiguration);
    this.bind('add', this.updateConfiguration);
    this.bind('remove', this.updateConfiguration);
  },
  modelId: function modelId(attrs) {
    return attrs.id;
  },
  updateConfiguration: function updateConfiguration() {
    this.configuration.set('links', this.toJSON());
  },
  addNewLink: function addNewLink() {
    var newLink = {
      id: this.length + 1,
      title: '',
      url: '',
      thumbnail: '',
      description: '',
      open_in_new_tab: 1
    };
    this.add(newLink);
    return this.get(this.length);
  }
});

ExternalLinkCollection.forContentElement = function (contentElement, entry) {
  return new ExternalLinkCollection(contentElement.configuration.get('links') || [], {
    entry: entry,
    configuration: contentElement.configuration
  });
};

var SidebarController = Marionette.Controller.extend({
  initialize: function initialize(options) {
    this.entry = options.entry;
    this.region = options.region;
  },
  link: function link(id, linkId) {
    var contentElement = this.entry.contentElements.get(id);
    var linksCollection = ExternalLinkCollection.forContentElement(contentElement, this.entry);
    this.region.show(new SidebarEditLinkView({
      model: linksCollection.get(linkId),
      collection: linksCollection,
      contentElement: contentElement
    }));
  }
});

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".SidebarListView-module_linksContainer__HvWq- {\n}\n";
var styles = {"linksContainer":"SidebarListView-module_linksContainer__HvWq-"};
styleInject(css);

var SidebarListView = Marionette.Layout.extend({
  template: function template(data) {
    return "\n    <div class='".concat(styles.linksContainer, "'></div>\n    <button class=\"").concat(buttonStyles.addButton, "\">\n      ").concat(I18n.t('pageflow_scrolled.editor.content_elements.externalLinkList.add'), "\n    </button>\n  ");
  },
  regions: cssModulesUtils.ui(styles, 'linksContainer'),
  events: cssModulesUtils.events(buttonStyles, {
    'click addButton': 'addElement'
  }),
  onRender: function onRender() {
    this.linksContainer.show(new ListView({
      collection: this.collection,
      onEdit: _.bind(this.onEdit, this),
      onRemove: _.bind(this.onRemove, this)
    }));
  },
  addElement: function addElement() {
    var newModel = this.collection.addNewLink();
    this.onEdit(newModel);
  },
  onEdit: function onEdit(linkModel) {
    editor.navigate("/scrolled/external_links/".concat(this.options.contentElement.id, "/").concat(linkModel.id), {
      trigger: true
    });
  },
  onRemove: function onRemove(linkModel) {
    if (window.confirm(I18n.t('pageflow_scrolled.editor.content_elements.externalLinkList.confirm_delete_link'))) {
      this.collection.remove(linkModel);
    }
  }
});

//router defines the URL hash path mapping and controller provides functions for the paths

editor.registerSideBarRouting({
  router: SidebarRouter,
  controller: SidebarController
}); // register external link list content element configuration editor for sidebar

editor.contentElementTypes.register('externalLinkList', {
  supportedPositions: ['inline'],
  configurationEditor: function configurationEditor(_ref) {
    var entry = _ref.entry;
    this.tab('general', function () {
      this.view(SidebarListView, {
        contentElement: this.model.parent,
        collection: ExternalLinkCollection.forContentElement(this.model.parent, entry)
      });
    });
  }
}); // register file handler for thumbnail of external link

editor.registerFileSelectionHandler('contentElement.externalLinks.link', function (options) {
  var contentElement = options.entry.contentElements.get(options.contentElementId);
  var links = ExternalLinkCollection.forContentElement(contentElement, options.entry);

  this.call = function (file) {
    var link = links.get(options.id);
    link.setReference('thumbnail', file);
  };

  this.getReferer = function () {
    return '/scrolled/external_links/' + contentElement.id + '/' + options.id;
  };
});

var DatawrapperAdView = Marionette.ItemView.extend({
  template: function template(data) {
    return "\n    <form action=\"https://datawrapper.de/chart/create\" method=\"POST\" target=\"_blank\">\n      <input type=\"hidden\" name=\"theme\" value=\"pageflow\" />\n      <input type=\"submit\" value=\"".concat(I18n.t('pageflow_scrolled.editor.content_elements.dataWrapperChart.attributes.create_chart.label'), "\" />\n    </form>\n  ");
  },
  className: 'datawrapper_ad'
});

editor.contentElementTypes.register('dataWrapperChart', {
  supportedPositions: ['inline', 'sticky', 'left', 'right', 'full'],
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.input('url', UrlInputView, {
        supportedHosts: ['http://cf.datawrapper.de', 'https://cf.datawrapper.de', 'http://datawrapper.dwcdn.de', 'https://datawrapper.dwcdn.de', 'http://datawrapper.dwcdn.net', 'https://datawrapper.dwcdn.net', 'http://charts.datawrapper.de', 'https://charts.datawrapper.de'],
        displayPropertyName: 'displayUrl',
        required: true,
        permitHttps: true
      });
      this.view(DatawrapperAdView);
      this.input('title', TextInputView, {
        placeholder: I18n.t('pageflow_scrolled.public.chart.default_title')
      });
      this.input('backgroundColor', ColorInputView, {
        defaultValue: '#323d4d'
      });
      this.group('ContentElementCaption');
      this.group('ContentElementPosition');
    });
  }
});

editor.contentElementTypes.register('vrImage', {
  configurationEditor: function configurationEditor() {
    this.tab('general', function () {
      this.input('image', FileInputView, {
        collection: 'image_files',
        fileSelectionHandler: 'contentElementConfiguration',
        filter: 'with_projection',
        positioning: false
      });
      this.input('initialYaw', SliderInputView$1, {
        unit: '°',
        minValue: -180,
        maxValue: 180
      });
      this.input('initialPitch', SliderInputView$1, {
        unit: '°',
        minValue: -60,
        maxValue: 60
      });
      this.group('ContentElementCaption');
      this.group('ContentElementPosition');
    });
  }
});
editor.fileTypes.modify('image_files', {
  configurationEditorInputs: function configurationEditorInputs(model) {
    var values = ['equirectangular_mono', 'equirectangular_stereo'];
    return [{
      name: 'projection',
      inputView: SelectInputView$1,
      inputViewOptions: {
        includeBlank: true,
        values: values
      }
    }];
  },
  confirmUploadTableColumns: [{
    name: 'projection',
    cellView: EnumTableCellView
  }],
  filters: [{
    name: 'with_projection',
    matches: function matches(file) {
      return !!file.configuration.get('projection');
    }
  }]
});
